/* *****************************************************************************
 *  Compilation:  javac Point2D.java
 *  Execution:    java Point2D x0 y0 n
 *  Dependencies: StdDraw.java StdRandom.java
 *
 *  Immutable point data type for points in the plane.
 ******************************************************************************/

import edu.princeton.cs.algs4.StdOut;
import edu.princeton.cs.algs4.StdRandom;

/**
 * This is an adapted version of the Algs4 Point2D class.
 * see <a href="https://algs4.cs.princeton.edu/12oop/Point2D.java.html">
 */
public class Point2D implements _____________ {

    private final double x;    // x coordinate
    private final double y;    // y coordinate

    /**
     * Initializes a new point (x, y).
     */
    public Point2D(double x, double y) {
        this.x = x;
        this.y = y;
    }

    /**
     * Returns the x-coordinate.
     */
    public double x() {
        return x;
    }

    /**
     * Returns the y-coordinate.
     */
    public double y() {
        return y;
    }

    /**
     * Return a string representation of this point in the format (x, y).
     */
    public String toString() {
        return "(" + x + ", " + y + ")";
    }

    /**
     * Returns the square of the Euclidean distance
     * between this point and that point.
     */
    public static double distanceSquaredTo(Point2D p, Point2D q) {
        double dx = p.x - q.x;
        double dy = p.y - q.y;
        return dx * dx + dy * dy;
    }

    /**
     * REQUIRED BY THE "COMPARABLE" INTERFACE.
     * Compares by y-coordinate, breaking ties by x-coordinate.
     * Returns 0 if this point is equal to that point, 1 if this point
     * is greater than that point and -1 if that point is greater than
     * this point.
     */
    public int compareTo(_____________) {

        //------ ADD CODE HERE ------//

        return 0;
    }

    /* ****************************** PART (B) ****************************** */
    /**
     * Returns a Comparator for comparing by distance to a reference point.
     */
    //    public static Comparator<Point2D> distanceToOrder(Point2D ref) {
    //        return ______________;
    //    }
    //
    //    // compare points according to their distance to a reference point.
    //    private static class DistanceToOrder implements _____________ {
    //
    //        private Point2D ref;
    //
    //        public DistanceToOrder(_____________) {
    //            _____________;
    //        }
    //
    //        // REQUIRED BY THE "COMPARATOR" INTERFACE.
    //        // Return 0  if distance between p and ref = distance between q and ref.
    //        // Return 1  if distance between p and ref > distance between q and ref.
    //        // Return -1 if distance between p and ref < distance between q and ref.
    //        public int _____________(_____________,_____________) {
    //
    //            //------ ADD CODE HERE ------//
    //        }
    //    }
    /* ****************************** PART (B) ****************************** */

    /**
     * Creates an array of random points and checks if they are all distinct.
     * This is done twice, each time with a different comparison method.
     */
    public static void main(String[] args) {
        int n = Integer.parseInt(args[0]);

        // Store n random points in an array.
        Point2D[] array = new Point2D[n];
        for (int i = 0; i < array.length; i++)
            array[i] = new Point2D(StdRandom.uniform(n + 1),
                    StdRandom.uniform(n + 1));

        StdOut.println("Before Sorting: ");
        for (Point2D p : array)
            StdOut.println(p);


        // Sort the points based on the y-coordinate, breaking ties using
        // The x-coordinate. (Hint: Use Arrays.sort)
        //------ ADD CODE HERE ------//


        StdOut.println("\nAfter Sorting: ");
        for (Point2D p : array)
            StdOut.println(p);


        // Check if the points in the array are distinct (in linear time
        // in the worst case).
        boolean distinct = true;
        //------ ADD CODE HERE ------//
        StdOut.println("Distinct: " + distinct);



        /* ****************************** PART (B) ****************************** */
        // Sort the points based on there distance to the origin (0,0).
        // (Hint: Use Arrays.sort)
        //------ ADD CODE HERE ------//
        //
        //
        //        StdOut.println("\nSorted by distance to the origin: ");
        //        for (Point2D p : array)
        //            StdOut.println(p);
        //
        //
        //        // Check if the points in the array array have distinct distances from
        //        // the origin (in linear time in the worst case).
        //        distinct = true;
        //        //------ ADD CODE HERE ------//
        //        StdOut.println("Distinct: " + distinct);
        /* ****************************** PART (B) ****************************** */
    }


}
